
MENU = "Core"
ICON = "menu"
ROLE = "core"


SUBMENUS = [
    {
        "menu": "Dashboard",
        "icon": "dashboard",
        "role": ROLE + "dashboard",
        "rota": ROLE + "dashboard",
    },
    {
        "menu": "Tipoentidade",
        "icon": "corporate_fare",
        "role": "list_tipoentidade",
        "rota": "list_tipoentidade",
    },    
    {
        "menu": "Entidades",
        "icon": "home",
        "role": "list_entidade",
        "rota": "list_entidade",
    },  
     
    {
        "menu": "Entidade",
        "icon": "home",
        "role": "view_entidade",
        "rota": "view_entidade",
    },  
    {
        "menu": "Sucursal",
        "icon": "house",
        "role": "list_sucursal",
        "rota": "list_sucursal",
    }, 
    {
        "menu": "Perfil",
        "icon": "groups",
        "role": "list_perfil",
        "rota": "list_perfil",
    }, 
    {
        "menu": "Usuario",
        "icon": "face",
        "role": "list_user",
        "rota": "list_usuario",
    },{
        "menu": "UsuarioAll",
        "icon": "people",
        "role": "list_user_all",
        "rota": "list_usuarioall",
    },  
    {
        "menu": "input",
        "icon": "input",
        "role": "list_input",
        "rota": "list_input",
    }, {
        "menu": "Cometario",
        "icon": "mode_comment",
        "role": "list_cometario",
        "rota": "list_cometario",
    }, 
    
]