<!DOCTYPE html>
<html>
<head>
  <meta charset="UTF-8">
  <title></title>
</head>
<body style="margin:0; padding:0; font-family:Arial, sans-serif; background-color:#f4f4f4;">
  <table width="100%" border="0" cellpadding="0" cellspacing="0">
   <tr >
      <td align="center" style="margin-top: 50px">
        <!-- LOGO -->
         <br><br><br>
      </td>
    </tr>
    <tr>
      <td align="center">
        <table width="100%" cellpadding="0" cellspacing="0" style="max-width: 600px; background-color: #ffffff; border-radius: 2px; overflow: hidden; box-shadow: 0 0 10px rgba(0,0,0,0.1);">
          
          <tr>
            <td style="padding: 40px; text-align: center;">
              <img src="{{logo}}" alt="Logo" style="height: 60px;">
              <h2 style="margin-top: 0; color: #333;">Bem Vindo {{nome}}</h2>
              <p style="font-size: 16px; color: #666;">
                {{ 'username' }}, clica no link abaixo para completar a verificacao do seu e-mail.
              </p>
              <p style="font-size: 30px; font-weight: bold; color: #000; margin: 10px 0;">
                {{ OTP_CODE }}
                <a class="btn btn-primary" href="{{link}}">
                  <h3><b>Aqui</b></h3>
                </a>
              </p>
              <p style="font-size: 14px; color: #999;">
                Este link expira em 5 minutos.
              </p>
            </td>
          </tr>
        </table>
      </td>
    </tr>
    <tr>
      <td align="center" style="padding: 30px 0;">
        <p style="font-size: 12px; color: #aaa; text-align: center;">
          © 2025 Sua Empresa. Todos os direitos reservados.<br>
          Não responda a este e-mail.
        </p>
      </td>
    </tr>
  </table>
</body>
</html>